#ifndef COMPAT_RCULIST_H
#define COMPAT_RCULIST_H

#include_next <linux/rculist.h>

/* Include the autogenerated header file */
#include "../../compat/config.h"

#ifdef HAVE_HLIST_FOR_EACH_ENTRY_3_PARAMS
#define compat_hlist_for_each_entry_rcu(pos, head, member) \
	hlist_for_each_entry_rcu(pos, head, member)

#else
#define compat_hlist_for_each_entry_rcu(pos, head, member) \
	hlist_for_each_entry_rcu(pos, hlnode, head, member)

#endif


#ifndef list_next_or_null_rcu
/**
 * list_next_or_null_rcu - get the first element from a list
 * @head:       the head for the list.
 * @ptr:        the list head to take the next element from.
 * @type:       the type of the struct this is embedded in.
 * @member:     the name of the list_head within the struct.
 *
 * Note that if the ptr is at the end of the list, NULL is returned.
 *
 * This primitive may safely run concurrently with the _rcu list-mutation
 * primitives such as list_add_rcu() as long as it's guarded by rcu_read_lock().
 */
#define list_next_or_null_rcu(head, ptr, type, member) \
({ \
        struct list_head *__head = (head); \
        struct list_head *__ptr = (ptr); \
        struct list_head *__next = READ_ONCE(__ptr->next); \
        likely(__next != __head) ? list_entry_rcu(__next, type, \
                                                  member) : NULL; \
})
#endif /* ! list_next_or_null_rcu */

#endif /* COMPAT_RCULIST_H */
